
import { write_text } from '../fs';
import { BuildState } from './state';
import { map_input_file_to_output_file } from './helpers';
// import { parseICS, CalendarComponent } from 'ical';
import create_calendar, { ICalEventData, ICalCalendarData } from 'ical-generator';
import { FrontMatterLocation } from '../template';

export type { ICalEventData, ICalCalendarData, ICalAttendeeData, ICalAttendeeStatus } from 'ical-generator';

export interface EventSeries {
	url: string;
	in_file: string;
	title?: string;
	description?: string;
	author_name?: string;
	author_email?: string;
	entries: {
		title?: string;
		start?: string;
		end?: string;
		time_zone?: `${string}/${string}`;
		location?: FrontMatterLocation;
	}[];
}

export interface EventEntry {
	url: string;
	in_file: string;
	title?: string;
	description?: string;
	author_name?: string;
	author_email?: string;
	start?: string;
	end?: string;
	time_zone?: `${string}/${string}`;
	location?: FrontMatterLocation;
}

export async function write_events_and_calendars_if_needed(state: BuildState) {
	if (state.conf.events) {
		for (const entry of state.events) {
			const event = icalendar_event(state, entry);
			const cal_data: ICalCalendarData = {
				prodId: {
					company: 'jbrumond.me',
					product: 'docs2website',
					language: 'EN',
				},
				// ...
			};

			const calendar = create_icalendar(cal_data, event);
			const out_file = await map_input_file_to_output_file(state, entry.in_file, [ '.html', '.md', '.markdown' ], '.ics');
			await write_text(out_file, calendar);
		}

		for (const series of state.event_series) {
			const events = series.entries.map((event) => {
				return icalendar_event(state, {
					url: series.url,
					in_file: series.in_file,
					title: event.title || series.title,
					description: series.description,
					author_name: series.author_name,
					author_email: series.author_email,
					start: event.start,
					end: event.end,
					time_zone: event.time_zone,
					location: event.location,
				});
			});
			
			const cal_data: ICalCalendarData = {
				name: series.title,
				description: series.description,
				url: series.url,
				prodId: {
					company: 'jbrumond.me',
					product: 'docs2website',
					language: 'EN',
				},
				// ...
			};

			const calendar = create_icalendar(cal_data, events);
			const out_file = await map_input_file_to_output_file(state, series.in_file, [ '.html', '.md', '.markdown' ], '.ics');
			await write_text(out_file, calendar);
		}
	}

	if (state.conf.calendars) {
		for (let index = 0; index < state.conf.calendars.length; index++) {
			const cal_conf = state.conf.calendars[index];
			const cal_entries = state.calendars[index];

			for (const entry of cal_entries) {
				const event = icalendar_event(state, entry);
				const cal_data: ICalCalendarData = {
					name: cal_conf.title,
					prodId: {
						company: 'jbrumond.me',
						product: 'docs2website',
						language: 'EN',
					},
					// ...
				};

				const calendar = create_icalendar(cal_data, event);
				await write_text(cal_conf.out_file, calendar);
			}
		}
	}
}

// export function parse_icalendar(contents: string) {
// 	const parsed = parseICS(contents);
// 	const calendar: CalendarComponent[] = [ ];

// 	for (const data of Object.values(parsed)) {
// 		calendar.push(data);
// 	}

// 	return calendar;
// }

export function create_icalendar(cal: ICalCalendarData, events: ICalEventData | ICalEventData[]) {
	const calendar = create_calendar(cal);

	if (Array.isArray(events)) {
		for (const event of events) {
			calendar.createEvent(event);
		}
	}

	else {
		calendar.createEvent(events);
	}

	return calendar.toString();
}

export function icalendar_event(state: BuildState, entry: EventEntry) : ICalEventData {
	const in_file = entry.in_file.slice(state.conf.input.root.length);
	const metadata = state.new_metadata.files[in_file];

	return {
		id: entry.url,
		summary: entry.title,
		description: entry.description || void 0,
		start: entry.start,
		end: entry.end,
		url: entry.url,
		timezone: entry.time_zone,
		created: metadata.first_seen_time,
		lastModified: metadata.last_updated_time,
		organizer: {
			name: entry.author_name || 'Unknown',
			email: entry.author_email,
		},
		location: format_location(entry.location),
		// attendees: post.mentions.flatMap((mention) : ICalAttendeeData | ICalAttendeeData[] => {
		// 	if (mention.is_rsvp && mention.is_reply_to_this) {
		// 		const ext = mention.external as ExternalEntry;
		// 		const status = ext.rsvp_type === 'yes'
		// 			? 'ACCEPTED' as const
		// 			: ext.rsvp_type === 'no'
		// 				? 'DECLINED' as const
		// 				: ext.rsvp_type === 'maybe'
		// 					? 'TENTATIVE' as const
		// 					: 'NEEDS-ACTION' as const;

		// 		return {
		// 			name: mention.author_name,
		// 			rsvp: ext.rsvp_type === 'yes',
		// 			status: status as ICalAttendeeStatus
		// 		};
		// 	}

		// 	return [ ];
		// }),
	};
}

function format_location(loc: FrontMatterLocation) {
	if (! loc) {
		return;
	}

	if (loc.description) {
		if (loc.address) {
			return `${loc.description} (${loc.address})`;
		}
		
		if (loc.lat && loc.long) {
			return `${loc.description} [${loc.lat}, ${loc.long}]`;
		}

		return loc.description;
	}

	if (loc.address) {
		return loc.address;
	}

	if (loc.lat && loc.long) {
		return `[${loc.lat}, ${loc.long}]`;
	}
}
